using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Web.UI.WebControls;
using System.Drawing;

namespace ImageGallery.Design
{
    class GalleryActionList : System.ComponentModel.Design.DesignerActionList
    {
        #region DATA MEMBERS

        Gallery _control = null;
        GalleryDesigner _parentDesigner = null;
        DesignerActionUIService _designerActionUISvc;

        #endregion

        #region CONSTRUCTOR

        public GalleryActionList(GalleryDesigner designer)
            : base(designer.Component)
        {
            _parentDesigner = designer;
            _control = (Gallery)_parentDesigner.Component;
            _designerActionUISvc = GetService(typeof(DesignerActionUIService)) as DesignerActionUIService;

        }

        #endregion

        #region PROPERTIES
               
        public bool ShowTitle
        {
            get
            {
                return _control.ShowTitle;
            }
            set
            {
                SetProperty("ShowTitle", value);
            }
        }

        public bool ShowDescription
        {
            get
            {
                return _control.ShowDescription;
            }
            set
            {
                SetProperty("ShowDescription", value);
            }
        }

        #endregion

        #region StyleValue

        #endregion
        
        #region Methods

        /// <summary>
        /// Function to show the auto format form
        /// </summary>
        private void ShowAutoFormat()
        {
            frmFormat frm = new frmFormat();
            if (frm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {

                SetStyle(frm.SelectedIndex);
            }

        }

        public void EditImages()
        {
            EditorServiceContext.EditValue(_parentDesigner, Component, "Images");         
           // _designerActionUISvc.Refresh(Component);
        }

        /// <summary>
        /// Function to set the property of the control
        /// </summary>
        /// <param name="name">name of the property</param>
        /// <param name="value">value of the property</param>
        private void SetProperty(string name, object value)
        {
            //TypeDescriptor.GetProperties(control)[name].SetValue(control, value);   
            PropertyDescriptor des = TypeDescriptor.GetProperties(_control)[name];
            des.SetValue(_control, value);
        }

        /// <summary>
        /// function to get the value of the property of control
        /// </summary>
        /// <param name="name">name of the property</param>
        /// <returns>value of the property</returns>
        private object GetProperty(string name)
        {
            PropertyDescriptor des = TypeDescriptor.GetProperties(_control)[name];
            return des.GetValue(_control);
        }

        /// <summary>
        /// Function to set style
        /// </summary>
        private void SetStyle(int index)
        {

            Style imageStyle = new Style();
            Style titleStyle = new Style();
            Style descriptionStyle = new Style();
            TableItemStyle navigationBarStyle = new TableItemStyle();
            TableStyle navigationItemBarStyle = new TableStyle();
            TableItemStyle navigationItemBarItemStyle = new TableItemStyle();
            TableItemStyle navigationItemBarHoverItemStyle = new TableItemStyle();
            TableItemStyle navigationItemBarSelectedItemStyle = new TableItemStyle();
             

            switch (index)
            {
                case 1:
                    {
                        #region SET BLUE STYLE

                        // IMAGE STYLE
                        imageStyle.BorderColor = Color.Black;
                        imageStyle.BorderStyle = BorderStyle.Solid;
                        imageStyle.BorderWidth = Unit.Parse("1px");

                        // TITLE STYLE
                        titleStyle.Font.Bold = true;
                        titleStyle.Font.Size = FontUnit.Parse("14px");

                        // DESCRIPTION STYLE
                        descriptionStyle.Font.Size = FontUnit.Parse("10px");

                        // NAVIGATION BAR STYLE
                        navigationBarStyle.HorizontalAlign = HorizontalAlign.Center;

                        // Navigation Item Bar Item Style
                        navigationItemBarItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarItemStyle.BorderColor = Color.Black;
                        navigationItemBarItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarItemStyle.BackColor = ColorTranslator.FromHtml("#5F74A3");
                        navigationItemBarItemStyle.HorizontalAlign = HorizontalAlign.Center; 

                        // Navigation Item Bar Hover Item Style
                        navigationItemBarHoverItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarHoverItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarHoverItemStyle.BorderColor = ColorTranslator.FromHtml("#8C9FCA");
                        navigationItemBarHoverItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarHoverItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarHoverItemStyle.BackColor = ColorTranslator.FromHtml("#5F74A3");
                        navigationItemBarHoverItemStyle.HorizontalAlign = HorizontalAlign.Center;

                        // Navigation Item Bar Selected Item Style
                        navigationItemBarSelectedItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarSelectedItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarSelectedItemStyle.BorderColor = Color.Black;
                        navigationItemBarSelectedItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarSelectedItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarSelectedItemStyle.BackColor = ColorTranslator.FromHtml("#8C9FCA");
                        navigationItemBarSelectedItemStyle.HorizontalAlign = HorizontalAlign.Center; 

                        #endregion

                        break;
                    } // END CASE 1

                case 2:
                    {
                        #region SET GRAY STYLE
                
                        // IMAGE STYLE
                        imageStyle.BorderColor = ColorTranslator.FromHtml("#E0E0E0");
                        imageStyle.BorderStyle = BorderStyle.Solid;
                        imageStyle.BorderWidth = Unit.Parse("1px");

                        // TITLE STYLE
                        titleStyle.Font.Bold = true;
                        titleStyle.Font.Size = FontUnit.Parse("14px");

                        // DESCRIPTION STYLE
                        descriptionStyle.Font.Size = FontUnit.Parse("10px");

                        // NAVIGATION BAR STYLE
                        navigationBarStyle.HorizontalAlign = HorizontalAlign.Center;
                        navigationBarStyle.BorderColor = ColorTranslator.FromHtml("#E0E0E0");
                        navigationBarStyle.BorderStyle = BorderStyle.Solid;
                        navigationBarStyle.BorderWidth = Unit.Parse("1px");
                        
                        // Navigation Item Bar Item Style
                        navigationItemBarItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarItemStyle.BorderColor = ColorTranslator.FromHtml("#E0E0E0");
                        navigationItemBarItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarItemStyle.BackColor = Color.White;
                        navigationItemBarItemStyle.HorizontalAlign = HorizontalAlign.Center; 

                        // Navigation Item Bar Hover Item Style
                        navigationItemBarHoverItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarHoverItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarHoverItemStyle.BorderColor = Color.Black;
                        navigationItemBarHoverItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarHoverItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarHoverItemStyle.BackColor = Color.White;
                        navigationItemBarHoverItemStyle.HorizontalAlign = HorizontalAlign.Center;

                        // Navigation Item Bar Selected Item Style
                        navigationItemBarSelectedItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarSelectedItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarSelectedItemStyle.BorderColor = Color.Gray;
                        navigationItemBarSelectedItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarSelectedItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarSelectedItemStyle.BackColor = ColorTranslator.FromHtml("#E0E0E0");
                        navigationItemBarSelectedItemStyle.HorizontalAlign = HorizontalAlign.Center; 

                        #endregion

                        break;
                    } // END CASE 2
                case 3:
                    {
                        #region SET BLACK STYLE

                        SetProperty("BackColor", Color.Black);

                        // IMAGE STYLE
                        imageStyle.BorderColor = Color.Empty;
                        imageStyle.BorderStyle = BorderStyle.None;
                        imageStyle.BorderWidth = Unit.Empty;

                        // TITLE STYLE
                        titleStyle.Font.Bold = true;
                        titleStyle.ForeColor = Color.White; 
                        titleStyle.Font.Size = FontUnit.Parse("14px");

                        // DESCRIPTION STYLE
                        descriptionStyle.Font.Size = FontUnit.Parse("10px");
                        descriptionStyle.ForeColor = Color.White;

                        // NAVIGATION BAR STYLE
                        navigationBarStyle.HorizontalAlign = HorizontalAlign.Center;
                        navigationBarStyle.BackColor = Color.White;  
                        navigationBarStyle.BorderColor = Color.Empty;
                        navigationBarStyle.BorderStyle = BorderStyle.None;
                        navigationBarStyle.BorderWidth = Unit.Empty;

                        // Navigation Item Bar Item Style
                        navigationItemBarItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarItemStyle.BorderColor = Color.Black; 
                        navigationItemBarItemStyle.BorderWidth = Unit.Parse("1px") ;
                        navigationItemBarItemStyle.BorderStyle = BorderStyle.Solid; 
                        navigationItemBarItemStyle.BackColor = Color.Silver;
                        navigationItemBarItemStyle.HorizontalAlign = HorizontalAlign.Center;

                        // Navigation Item Bar Hover Item Style
                        navigationItemBarHoverItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarHoverItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarHoverItemStyle.BorderColor = Color.Silver;
                        navigationItemBarHoverItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarHoverItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarHoverItemStyle.BackColor = Color.Silver;
                        navigationItemBarHoverItemStyle.HorizontalAlign = HorizontalAlign.Center;

                        // Navigation Item Bar Selected Item Style
                        navigationItemBarSelectedItemStyle.Width = Unit.Parse("20px");
                        navigationItemBarSelectedItemStyle.Height = Unit.Parse("20px");
                        navigationItemBarSelectedItemStyle.BorderColor = Color.Silver;
                        navigationItemBarSelectedItemStyle.BorderWidth = Unit.Parse("1px");
                        navigationItemBarSelectedItemStyle.BorderStyle = BorderStyle.Solid;
                        navigationItemBarSelectedItemStyle.BackColor = Color.Black;
                        navigationItemBarSelectedItemStyle.ForeColor = Color.White; ;
                        navigationItemBarSelectedItemStyle.HorizontalAlign = HorizontalAlign.Center;

                        #endregion

                        break;
                    } // END CASE 3

            } // END SWITCH

            _control.Font.Name = "verdana";
            SetProperty("ImageStyle", imageStyle);
            SetProperty("TitleStyle", titleStyle);
            SetProperty("DescriptionStyle", descriptionStyle);
            SetProperty("NavigationBarStyle", navigationBarStyle);
            SetProperty("NavigationItemBarStyle", navigationItemBarStyle);
            SetProperty("NavigationItemBarItemStyle", navigationItemBarItemStyle);
            SetProperty("NavigationItemBarHoverItemStyle", navigationItemBarHoverItemStyle);
            SetProperty("NavigationItemBarSelectedItemStyle", navigationItemBarSelectedItemStyle);

            //_parentDesigner.UpdateDesignTimeHtml();
            //RefreshAll();
        }
        
        /// <summary>
        /// function to refresh the contents at the design time at the control
        /// </summary>
        private void RefreshAll()
        {            
            _designerActionUISvc.Refresh(_control);
            _parentDesigner.UpdateDesignTimeHtml();             

        }

        #endregion

        #region OVERRIDED METHODS

        /// <summary>
        /// Function to get the sorted action list.
        /// </summary>
        /// <returns></returns>
        public override DesignerActionItemCollection GetSortedActionItems()
        {

            DesignerActionItemCollection list = new DesignerActionItemCollection();
            list.Add(new DesignerActionMethodItem(this, "ShowAutoFormat", "Auto Format...", "Format"));            
            list.Add(new DesignerActionMethodItem(this, "EditImages", "Images...", "images"));
            list.Add(new DesignerActionPropertyItem("ShowTitle", "Show Title", "Other"));
            list.Add(new DesignerActionPropertyItem("ShowDescription", "Show Description", "Other"));

            return list;
        }

        #endregion

    }
}
