using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.IO;


namespace ImageGallery
{
    //[PersistChildren(false)]
    //[ParseChildren(true, "Images")]
    [ParseChildren(true)]
    [PersistChildren(false)]
    [ToolboxData("<{0}:Gallery runat=server></{0}:Gallery>")]
    [Designer(typeof(ImageGallery.Design.GalleryDesigner))]
    public class Gallery : WebControl
    {

        #region DATA MEMBERS

        string _imageFolder = string.Empty;
        string _imagesExtensions = "*.gif, *.jpg, *.bmp";
        bool _showTitle = false;
        bool _showDescription = false;
        bool _showNavigationBar = true;
        bool _autoNext = false;
        int _autoNextAfter = 5;
        bool _showNextButton = true;
        bool _showPreviousButton = true;
        string _nextButtonText = ">";
        string _previousButtonText = "<";


        int _selecteditemIndex = -1;

        object _dataSource = null;
        string _dataImageField = string.Empty;
        string _dataTitleField = string.Empty;
        string _dataDescriptionField = string.Empty;

        ImageCollection _images = null;

        Style _imageStyle = null;
        Style _titleStyle = null;
        Style _descriptionStyle = null;

        TableItemStyle _navigationBarStyle = null;
        TableStyle _navigationItemBarStyle = null;
        TableItemStyle _navigationItemBarItemStyle = null;
        TableItemStyle _navigationItemBarHoverItemStyle = null;
        TableItemStyle _navigationItemBarSelectedItemStyle = null;

        private ImageChangeStyle _imageChangeStyle = ImageChangeStyle.Simple;

        internal string CssClassName = string.Empty; // Css class name for Navigation item
        internal string CssHoverClassName = string.Empty; // Css class name for Navigation hover item
        internal string CssSelectedClassName = string.Empty; // Css class name for Navigation selected item

        #endregion

        #region PROPERTIES

        #region DATA

        /// <summary>
        /// Property to get or set datasource
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Category("Data")]
        [Description("Get or set the data source")]
        public object DataSource
        {
            get { return _dataSource; }
            set
            {
                if ((value == null) || (value is IListSource) || (value is IEnumerable))
                {
                    _dataSource = value;

                }
                else
                {
                    throw new ArgumentException();
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        [Category("Data")]
        [Description("Field name of data source which contains URL of the image.")]
        public string DataImageField
        {
            get
            {

                return (_dataImageField) ?? string.Empty;
            }
            set
            {
                _dataImageField = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        [Category("Data")]
        [Description("Field name of data source which contains Title of the image.")]
        public string DataTitleField
        {
            get
            {

                return (_dataTitleField) ?? string.Empty;
            }
            set
            {
                _dataTitleField = value;
            }
        }
        /// <summary>
        /// Property to get or Set Data desription field
        /// </summary>
        [Category("Data")]
        [Description("Field name of data source which contains description of the image.")]
        public string DataDescriptionField
        {
            get
            {

                return (_dataDescriptionField) ?? string.Empty;
            }
            set
            {
                _dataDescriptionField = value;
            }
        }

        #endregion

        #region BEHAVIOUR
                
        [Category("Behavior")]
        [Description("true for show image description.")]
        public ImageChangeStyle ImageChangeStyle
        {
            get { return _imageChangeStyle; }
            set { _imageChangeStyle = value; }
        }

        [Category("Behavior")]
        [Description("true for automatically select next image after certaint period of time.")]
        public bool AutoNext
        {
            get
            {
                return _autoNext;
            }
            set
            {
                _autoNext = value;
            }
        }

        [Category("Behavior")]
        [Description("Time inverval in seconds for auto next image. ")]
        public int AutoNextAfter
        {
            get
            {
                return _autoNextAfter;
            }
            set
            {
                if (value > 1)
                {
                    _autoNextAfter = value;
                }
                else
                {
                    throw new Exception("Interval should be greater then 1.");
                }
            }
        }

        
        #endregion

        #region Appearance

        [Category("Appearance")]
        [Description("true for show image title.")]
        public bool ShowTitle
        {
            get
            {
                return _showTitle;
            }
            set
            {
                _showTitle = value;
            }
        }

        [Category("Appearance")]
        [Description("true for show image description.")]
        public bool ShowDescription
        {
            get
            {
                return _showDescription;
            }
            set
            {
                _showDescription = value;
            }
        }

        [Category("Appearance")]
        [Description("true for show navigation description.")]
        public bool ShowNavigationBar
        {
            get
            {
                return _showNavigationBar;
            }
            set
            {
                _showNavigationBar = value;
            }
        }

        [Category("Appearance")]
        [Description("true for show next image button in navigation bar.")]
        public bool ShowNextButton
        {
            get
            {
                return _showNextButton;
            }
            set
            {
                _showNextButton = value;
            }
        }

        [Category("Appearance")]
        [Description("true for show previous image button in navigation bar.")]
        public bool ShowPreviousButton
        {
            get
            {
                return _showPreviousButton;
            }
            set
            {
                _showPreviousButton = value;
            }
        }

        [Category("Appearance")]
        [Description("get or set the text of the next button.")]
        public string NextButtonText
        {
            get
            {
                return _nextButtonText ?? string.Empty;
            }
            set
            {
                _nextButtonText = value;
            }
        }

        [Category("Appearance")]
        [Description("get or set the text of the previous button.")]
        public string PreviousButtonText
        {
            get
            {
                return _previousButtonText ?? string.Empty;
            }
            set
            {
                _previousButtonText = value;
            }
        }

        #endregion

        #region STYLE

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style of Image")]
        public Style ImageStyle
        {
            get
            {
                if (_imageStyle == null)
                    _imageStyle = new Style();
                return _imageStyle;
            }
            set
            {
                _imageStyle = value;
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style of image Title")]
        public Style TitleStyle
        {
            get
            {
                if (_titleStyle == null)
                    _titleStyle = new Style();
                return _titleStyle;
            }
            set
            {
                _titleStyle = value;
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style of image description")]
        public Style DescriptionStyle
        {
            get
            {
                if (_descriptionStyle == null)
                    _descriptionStyle = new Style();
                return _descriptionStyle;
            }
            set
            {
                _descriptionStyle = value;
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style of Navigational bar.")]
        public TableItemStyle NavigationBarStyle
        {
            get
            {
                if (_navigationBarStyle == null)
                    _navigationBarStyle = new TableItemStyle();

                return _navigationBarStyle;
            }
            set { _navigationBarStyle = value; }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style of Navigational Item bar.")]
        public TableStyle NavigationItemBarStyle
        {
            get
            {
                if (_navigationItemBarStyle == null)
                    _navigationItemBarStyle = new TableStyle();

                return _navigationItemBarStyle;
            }
            set { _navigationItemBarStyle = value; }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style for the item of the Navigational item bar.")]
        public TableItemStyle NavigationItemBarItemStyle
        {
            get
            {
                if (_navigationItemBarItemStyle == null)
                    _navigationItemBarItemStyle = new TableItemStyle();

                return _navigationItemBarItemStyle;
            }
            set { _navigationItemBarItemStyle = value; }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style for the item of the Navigational hover item bar.")]
        public TableItemStyle NavigationItemBarHoverItemStyle
        {
            get
            {
                if (_navigationItemBarHoverItemStyle == null)
                    _navigationItemBarHoverItemStyle = new TableItemStyle();

                return _navigationItemBarHoverItemStyle;
            }
            set
            {
                _navigationItemBarHoverItemStyle = value;
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Category("Styles")]
        [Description("Style for the item of the Navigational hover item bar.")]
        public TableItemStyle NavigationItemBarSelectedItemStyle
        {
            get
            {
                if (_navigationItemBarSelectedItemStyle == null)
                    _navigationItemBarSelectedItemStyle = new TableItemStyle();

                return _navigationItemBarSelectedItemStyle;
            }
            set
            {
                _navigationItemBarSelectedItemStyle = value;
            }
        }

        #endregion
        
        #region MISC

        /// <summary>
        /// Get or Set the path of the image folder.
        /// </summary>
        [Description("Path of the image folder.")]
        [Editor(typeof(ImageGallery.Design.ImageFolderEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public string ImageFolder
        {
            get
            {
                return (_imageFolder != null) ? _imageFolder : string.Empty;
            }
            set
            {
                _imageFolder = value;
            }
        }

        /// <summary>
        /// Get or Set the Image extension which you want to display from image folder.
        /// </summary>
        [Description("Extension of those images which you want to display from image Folder.")]
        public string ImagesExtensions
        {
            get
            {
                return _imagesExtensions ?? string.Empty;
            }
            set
            {
                _imagesExtensions = value;
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]  // Content generates code for each page
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Description("Images")]
        public ImageCollection Images
        {
            get
            {
                if (_images == null)
                    _images = new ImageCollection();
                return _images;
            }
        }

        #endregion

        #endregion

        #region OVERRIDED PROPERTY

        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                return HtmlTextWriterTag.Div;
            }
        }

        #endregion

        #region OVERRIDED METHODS

        protected override void OnInit(EventArgs e)
        {
            JavaScript.Gallery = this; 
            
            CssClassName = this.ClientID + "_0";
            CssHoverClassName = this.ClientID + "_1";
            CssSelectedClassName = this.ClientID + "_2";

            
            
            

            if (!Page.ClientScript.IsClientScriptIncludeRegistered("imageRotator"))
            {
                Page.ClientScript.RegisterClientScriptInclude(this.GetType(), "imageRotator", Page.ClientScript.GetWebResourceUrl(this.GetType(), "ImageGallery.res.ImageRotator.js"));
            }
            base.OnInit(e);
        }

        protected override void OnPreRender(EventArgs e)
        {

            // IF BOTH STYLES ARE NOT EMPTY
            if (!NavigationItemBarItemStyle.IsEmpty || !NavigationItemBarHoverItemStyle.IsEmpty || !NavigationItemBarSelectedItemStyle.IsEmpty)
            {

                string styleID = this.UniqueID + "_style";

                System.Web.UI.HtmlControls.HtmlGenericControl style =
                (System.Web.UI.HtmlControls.HtmlGenericControl)this.Page.Header.FindControl(styleID);


                if (style == null)
                {
                    style = new System.Web.UI.HtmlControls.HtmlGenericControl("style");
                    style.Attributes.Add("type", "text/css");
                    style.ID = styleID;

                    StringBuilder cssClass = new StringBuilder();

                    // ITEM STYLE
                    if (!NavigationItemBarItemStyle.IsEmpty)
                    {
                        cssClass.Append(GetCssClass(CssClassName, NavigationItemBarItemStyle) + "\n");
                    }

                    // ITEM HOVER STYLE
                    if (!NavigationItemBarHoverItemStyle.IsEmpty)
                    {
                        cssClass.Append(GetCssClass(CssHoverClassName, NavigationItemBarHoverItemStyle));
                    }

                    // ITEM SELECTED STYLE
                    if (!NavigationItemBarSelectedItemStyle.IsEmpty)
                    {
                        cssClass.Append(GetCssClass(CssSelectedClassName, NavigationItemBarSelectedItemStyle));
                    }

                    style.InnerText = cssClass.ToString();


                    this.Page.Header.Controls.Add(style);

                }
            }
            base.OnPreRender(e);
        }

        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            // GENERATE CLASS FOR NAVIGATIONAL ITEM.
            // GenerateCssClass(writer);


            writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "table");

            if (this.DesignMode)
            {
                if (this.Images.Count == 0)
                {
                    writer.AddStyleAttribute(HtmlTextWriterStyle.Padding, "2px");
                    //if (this.Width.IsEmpty)
                    writer.AddStyleAttribute(HtmlTextWriterStyle.Width, "300px");
                }
            }

            base.AddAttributesToRender(writer);
        }

        protected override void RenderContents(HtmlTextWriter output)
        {
            BindImagesFromFolder();

            Image selImage = null;

            output.RenderBeginTag(HtmlTextWriterTag.Div);
            if (this.DesignMode)
            {
                GetSelectedImage(out selImage);
                AddImageControl(output, selImage, true);
            }
            else
            {
                // GENERATE JAVASCRIPT
                GenerateJavascript(output, out selImage);

                // GENERATE IMAGES                
                foreach (Image image in Images)
                {
                    bool selected = (image.Index == selImage.Index);
                    AddImageControl(output, image, selected);
                }
            }
            output.RenderEndTag();

            #region DRAW BAR

            if (ShowNavigationBar)
            {
                int selIndex = (selImage == null) ? -1 : selImage.Index;
                DrawNavigationalBar(output, selIndex);
            }

            #endregion

            #region JAVASCRIPT FOR SELECTED ITEM AND IF AUTO NEXT


            //***** Add SELECTED ITEM Refference
            if (!this.DesignMode && selImage != null)
            {
                output.Write("<script type='text/javascript'>");

                #region Navigation Bar Javascript

                if (ShowNavigationBar)
                {
                    output.Write(JavaScript.SelectedItemObjectName + " = document.getElementById('" + selImage.NavigationBarItemID + "');");

                    // Next Button
                    if (ShowNextButton)
                    {
                        output.Write(JavaScript.GetNextMethod());
                    }

                    // Previous Button
                    if (ShowPreviousButton)
                    {
                        output.Write(JavaScript.GetPreviousMethod());
                    }

                }

                #endregion

                output.Write(JavaScript.ImageRotatorObjectName + ".selectedIndex = " + selImage.Index.ToString() + ";");
                output.Write(JavaScript.ImageRotatorObjectName + ".selectedImage = document.getElementById('" + selImage.ContainerID + "');");

                #region Auto Next Javascript

                // AUTO NEXT FUNCTION
                if (AutoNext)
                {
                    output.Write(JavaScript.GetAutoNextMethod() );
                }

                #endregion

                output.Write("</script>");
            }

            #endregion

        }

        public override void DataBind()
        {
            //base.DataBind();
            BindImagesFromDataSource();
        }

        #endregion

        #region PUBLIC METHODS



        #endregion

        #region PRIVATE METHODS


        /// <summary>
        /// Function to get selected image and generate the javascript for control.
        /// </summary>
        /// <param name="output"></param>
        /// <param name="selImage">for returning the selected image</param>
        private void GenerateJavascript(HtmlTextWriter output, out Image selImage)
        {
            StringBuilder strJava = new StringBuilder();

            selImage = (this.Images.Count > 0) ? (Image)this.Images[0] : null;
                        
            strJava.Append("<script type='text/javascript'>");

            #region Populate JS Objects

            strJava.Append("var " + JavaScript.ImageRotatorObjectName + " = new imageRotator('" + this.ClientID + "' , 200,200);");
            strJava.Append("var " + JavaScript.SelectedItemObjectName + " = null;");
            strJava.Append("var img1;");

            string strFormat = "img1 = new image(\"{0}\", \"{1}\",\"{2}\",\"{3}\");" +
                               JavaScript.ImageRotatorObjectName + ".addImage(img1);";

            foreach (Image image in this.Images)
            {

                image.ContainerID = this.ClientID + "_ImageContainer" + image.Index.ToString();
                image.NavigationBarItemID = this.ClientID + "_button" + image.Index.ToString();


                strJava.Append(string.Format(strFormat, image.ContainerID, image.NavigationBarItemID, image.Value, image.Name));

                if (image.Selected)
                {
                    selImage = image;
                }
            } // END FOR LOOP

            #endregion


            // SELECT IMAGE FUNCTION
            strJava.Append(JavaScript.GetSelectImageFunction() );


            if (this.ShowNavigationBar)
            {
                // JAVA SCRIPT METHODS FOR SETTING STYLE
                strJava.Append(JavaScript.GetMouseOutFunction());
                strJava.Append(JavaScript.GetMouseOverFunction());

                // FUNCTION FOR SELCTED STYLE
                strJava.Append(JavaScript.GetMouseClickFunction());
            }
            
            strJava.Append("</script>");

            output.Write(strJava.ToString());
        }

        /// <summary>
        /// Function to add Image Control
        /// </summary>
        /// <param name="output">html text writer</param>
        /// <param name="image">image</param>
        /// <param name="isSelected">true for selected image</param>
        private void AddImageControl(HtmlTextWriter output, Image image, bool isSelected)
        {
            output.AddAttribute(HtmlTextWriterAttribute.Id, image.ContainerID);
            output.AddAttribute(HtmlTextWriterAttribute.Name, image.ContainerID);
            output.AddAttribute(HtmlTextWriterAttribute.Width, "100%");
            output.AddStyleAttribute(HtmlTextWriterStyle.Width, "100%");
            //output.AddStyleAttribute(HtmlTextWriterStyle.Position, "relative");
            //output.AddStyleAttribute(HtmlTextWriterStyle.ZIndex, "-1");

            string imgURL = image.ImageUrl;
            // SET SELECTED IMAGE URL
            if (isSelected)
            {
                output.AddStyleAttribute(HtmlTextWriterStyle.Display, "block");
                if (this.DesignMode && imgURL != string.Empty)
                {
                    imgURL = GetDesignTimeImagePath(imgURL);
                }

            }
            else
            {
                output.AddStyleAttribute(HtmlTextWriterStyle.Display, "none");
            }


            output.RenderBeginTag(HtmlTextWriterTag.Div);

            #region DRAW IMAGE

            //output.RenderBeginTag(HtmlTextWriterTag.Div);
            System.Web.UI.WebControls.Image imageControl = new System.Web.UI.WebControls.Image();
            imageControl.ID = this.ClientID + "_Image" + image.Index.ToString();
            imageControl.ImageUrl = imgURL;


            if (!ImageStyle.IsEmpty) // IF STYLE IS NOT EMPTY THEN ASSIGN THIS STYLE TO THE CONTROL
                imageControl.ApplyStyle(ImageStyle);

            imageControl.RenderControl(output);

            //output.RenderEndTag();

            #endregion

            #region Image Title

            if (ShowTitle)
            {
                DrawTitle(output, image);
            }

            #endregion

            #region IMAGE DESCRIPTION

            if (ShowDescription)
            {
                DrawDescription(output, image);
            }

            #endregion

            //output.Write("</div>");
            output.RenderEndTag();
        }

        /// <summary>
        /// Functiont to Get the images from Folder and add them in to Images list.
        /// </summary>
        private void BindImagesFromFolder()
        {
            if (this.DesignMode) return;

            if (ImageFolder.Length == 0) return; // IF no folder path then return

            try
            {
                string path = GetPhysicalFolderPath(ImageFolder);
                string[] extensions = ImagesExtensions.Split(',');
                if (extensions == null || extensions.Length == 0)
                    extensions = new string[] { "*.jpg" };

                foreach (string ext in extensions)
                {
                    string[] files = Directory.GetFiles(path, ext.Trim(), SearchOption.TopDirectoryOnly);
                    //  string[] files = Directory.GetFiles(path);
                    foreach (string file in files)
                    {
                        string name = Path.GetFileName(file);
                        string value = name;
                        string imageUrl = ImageFolder.Replace('\\', '/') + "/" + name;
                        string title = "";
                        string description = "";
                        bool selected = false;

                        AddImage(name, value, imageUrl, title, description, selected);
                    }
                }
            }
            catch (DirectoryNotFoundException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Function to bind images from datasource
        /// </summary>
        private void BindImagesFromDataSource()
        {
            if (this.DesignMode) return;

            IEnumerable enumDataSource = GetDataSource();
            if ((enumDataSource == null) || (DataImageField == string.Empty)) return;

            foreach (object dataItem in enumDataSource)
            {
                PropertyDescriptorCollection propertyDesc = TypeDescriptor.GetProperties(dataItem);

                string name = "";// Path.GetFileName(file);
                string value = "";// name;

                // Image URL
                string imageUrl = "";
                if (DataImageField != string.Empty)
                {
                    if (propertyDesc[DataImageField] == null)
                    {
                        throw new ArgumentException("Invalid field name " + DataImageField);
                    }

                    imageUrl = propertyDesc[DataImageField].GetValue(dataItem).ToString();
                }

                // TITLE
                string title = "";
                if (DataTitleField != string.Empty)
                {
                    if (propertyDesc[DataTitleField] == null)
                    {
                        throw new ArgumentException("Invalid field name " + DataTitleField);
                    }

                    title = propertyDesc[DataTitleField].GetValue(dataItem).ToString();
                }
                // DESCRIPTION
                string description = "";
                if (DataDescriptionField != string.Empty)
                {
                    if (propertyDesc[DataDescriptionField] == null)
                    {
                        throw new ArgumentException("Invalid field name " + DataDescriptionField);
                    }

                    description = propertyDesc[DataDescriptionField].GetValue(dataItem).ToString();
                }

                // SELECTED
                bool selected = false;

                AddImage(name, value, imageUrl, title, description, selected);

            }


        }

        /// <summary>
        /// Function to get data source
        /// </summary>
        /// <returns>IEnumberable data source</returns>
        private IEnumerable GetDataSource()
        {
            if (DataSource == null)
                return null;

            IEnumerable resolvedDataSource = DataSource as IEnumerable;
            if (resolvedDataSource != null)
            {
                return resolvedDataSource;
            }


            IListSource listSource = DataSource as IListSource;
            if (listSource != null)
            {
                IList list = listSource.GetList();
                return (IEnumerable)list;
            }

            return null;
        }

        /// <summary>
        /// Function to draw Navigational bar.
        /// </summary>
        /// <param name="output"></param>
        /// <param name="selectedIndex">selected Image Index</param>
        private void DrawNavigationalBar(HtmlTextWriter output, int selectedIndex)
        {
            if (!NavigationBarStyle.IsEmpty)
            {
                NavigationBarStyle.AddAttributesToRender(output);
            }

            output.RenderBeginTag(HtmlTextWriterTag.Div);

            if (!NavigationItemBarStyle.IsEmpty)
            {
                NavigationItemBarStyle.AddAttributesToRender(output);
            }
            output.RenderBeginTag(HtmlTextWriterTag.Table);
            output.RenderBeginTag(HtmlTextWriterTag.Tr);

            #region PREVIOUS BUTTON

            if (ShowPreviousButton)
            {
                DrawNavigationBarItem(output, this.UniqueID + "_Pre", PreviousButtonText, -1, NavigationBarButtonType.Previous, false);
            }

            #endregion

            #region IMAGE INDEX BUTTONS

            int count = Images.Count;
            if (this.DesignMode && Images.Count == 0) // IF DESIGN MODE AND NO IMAGE IS ADDED THEN
            {
                // DrawNavigationBarItem(output, 0, true);
                count = 3;
                selectedIndex = 0;
            }

            for (int i = 0; i < count; i++)
            {
                string itemID = (i < Images.Count) ? Images[i].NavigationBarItemID : string.Empty;
                DrawNavigationBarItem(output, itemID, i.ToString(), i, NavigationBarButtonType.Simple, (i == selectedIndex));

            }

            #endregion

            #region NEXT BUTTON

            if (ShowNextButton)
            {
                DrawNavigationBarItem(output, this.UniqueID + "_Next", NextButtonText , -1, NavigationBarButtonType.Next, false);
            }

            #endregion

            output.RenderEndTag(); // END FOR TR
            output.RenderEndTag(); // END FOR TABLE
            output.RenderEndTag(); // END OF DIV

        }

       /// <summary>
        /// Function to Draw the navigation bar item
       /// </summary>
       /// <param name="output"></param>
       /// <param name="itemID"></param>
       /// <param name="text"></param>
       /// <param name="index"></param>
       /// <param name="buttonType"></param>
       /// <param name="isSelected"></param>
        private void DrawNavigationBarItem(HtmlTextWriter output, string itemID, string text, int index, NavigationBarButtonType buttonType, bool isSelected)
        {
            string sCss = string.Empty;

            if (isSelected && !NavigationItemBarSelectedItemStyle.IsEmpty)
            {
                if (this.DesignMode) // RENDER STYLE IF IT IS DESIGN MODE
                {
                    NavigationItemBarSelectedItemStyle.AddAttributesToRender(output);
                }
                sCss = CssSelectedClassName;
            }
            else if (!NavigationItemBarItemStyle.IsEmpty)
            {
                if (this.DesignMode)
                {
                    NavigationItemBarItemStyle.AddAttributesToRender(output);
                }
                sCss = CssClassName;
            }


            output.AddAttribute(HtmlTextWriterAttribute.Id, itemID);
            output.AddAttribute(HtmlTextWriterAttribute.Name, itemID);
            output.AddStyleAttribute(HtmlTextWriterStyle.Cursor, "pointer");

            switch(buttonType)
            {
                case NavigationBarButtonType.Next:
                    {
                        output.AddAttribute(HtmlTextWriterAttribute.Onclick, "javascript:" + JavaScript.NextImageMethodName + "()");
                        break;
                    }
                case NavigationBarButtonType.Previous:
                    {
                        output.AddAttribute(HtmlTextWriterAttribute.Onclick, "javascript:" + JavaScript.PreviousImageMethodName + "()");
                        break;
                    }
                default:
                    {
                        output.AddAttribute(HtmlTextWriterAttribute.Onclick, "javascript:" + JavaScript.ChangeImageMethodName + "(" + index + ", this)");
                        break;
                    }
            }

            if (sCss != string.Empty)
            {
                output.AddAttribute(HtmlTextWriterAttribute.Class, sCss);
            }

            if (!NavigationItemBarHoverItemStyle.IsEmpty)
            {
                output.AddAttribute("onmouseover", "javascript:" + JavaScript.MouseOverMethodName  + "(this)");
                output.AddAttribute("onmouseout", "javascript:" + JavaScript.MouseOutMethodName + "(this)");
            }



            output.RenderBeginTag(HtmlTextWriterTag.Td);
            output.Write(text);
            output.RenderEndTag(); // END FOR TD
        }

        /// <summary>
        /// Function to draw title
        /// </summary>
        /// <param name="output">out put stream</param>
        /// <param name="selectedImage">selected image</param>
        private void DrawTitle(HtmlTextWriter output, Image selectedImage)
        {
            if (selectedImage == null) return;

            if (!TitleStyle.IsEmpty)
                TitleStyle.AddAttributesToRender(output);

            //output.AddAttribute(HtmlTextWriterAttribute.Id, TitleControlID);
            //output.AddAttribute(HtmlTextWriterAttribute.Name, TitleControlID);
            output.RenderBeginTag(HtmlTextWriterTag.Div);
            output.Write(selectedImage.Title);
            output.RenderEndTag();
        }

        /// <summary>
        /// Function to draw Description of image
        /// </summary>
        /// <param name="output">out put stream</param>
        /// <param name="selectedImage">selected image</param>
        private void DrawDescription(HtmlTextWriter output, Image selectedImage)
        {
            if (selectedImage == null) return;

            if (!DescriptionStyle.IsEmpty)
                DescriptionStyle.AddAttributesToRender(output);

            //output.AddAttribute(HtmlTextWriterAttribute.Id, DescriptionControlID);
            //output.AddAttribute(HtmlTextWriterAttribute.Name, DescriptionControlID);
            output.RenderBeginTag(HtmlTextWriterTag.Div);
            output.Write(selectedImage.Description);
            output.RenderEndTag();
        }

        /// <summary>
        /// Function to add image in the images list.
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value"></param>
        /// <param name="imageUrl"></param>
        /// <param name="title"></param>
        /// <param name="description"></param>
        /// <param name="selected"></param>
        private void AddImage(string name, string value, string imageUrl, string title, string description, bool selected)
        {
            Image img = new Image();
            img.Description = description;
            img.ImageUrl = imageUrl;
            img.Name = name;
            img.Selected = selected;
            img.Title = title;
            img.Value = value;
            img.Index = Images.Count;
            Images.Add(img);
        }

        /// <summary>
        /// Function to get Physical path of the image
        /// </summary>
        /// <param name="imageUrl">image url</param>
        /// <returns>physical path of image</returns>
        private string GetDesignTimeImagePath(string imageUrl)
        {
            if (this.DesignMode)
            {
                try
                {
                    IComponent component = ((IComponent)this);
                    IWebApplication webApp = (IWebApplication)component.Site.GetService(typeof(IWebApplication));

                    IProjectItem image = webApp.GetProjectItemFromUrl(imageUrl);

                    return image.PhysicalPath;
                }
                catch
                {
                    return imageUrl;
                }
            }

            return imageUrl;

        }

        /// <summary>
        /// Function to get the Physical folder path.
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        private string GetPhysicalFolderPath(string path)
        {
            if (this.DesignMode) return string.Empty;

            string strToRetrun = string.Empty;
            if (path.IndexOf("~\\") == 0 || path.IndexOf("~/") == 0)
            {
                strToRetrun = HttpContext.Current.Request.PhysicalApplicationPath + path.Remove(0, 2);
            }
            else
            {
                strToRetrun = System.IO.Path.GetDirectoryName(HttpContext.Current.Request.PhysicalPath) + System.IO.Path.DirectorySeparatorChar + path;
            }
            return strToRetrun;
        }

        /// <summary>
        /// Function to get the first selected image from images list.
        /// </summary>
        /// <param name="selImage"></param>
        private void GetSelectedImage(out Image selImage)
        {

            if (this.DesignMode && this.Images.Count == 0)
            {
                selImage = new Image();

                selImage.Name = "Image1";
                selImage.Value = "Image1";
                selImage.ImageUrl = this.Page.ClientScript.GetWebResourceUrl(this.GetType(), "ImageGallery.res.logo.jpg");
                selImage.Title = "Title";
                selImage.Description = "Description of the image will be here...";
                selImage.Selected = true;
                //selImage.Index                
            }
            else
            {
                selImage = (this.Images.Count > 0) ? (Image)this.Images[0] : null;
                foreach (Image image in this.Images)
                {
                    if (image.Selected)
                    {
                        selImage = image;
                        break;
                    }
                } // END FOR LOOP
            }

        }

        /// <summary>
        /// Function the get the URL of image
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        private string GetImageUrl(string url)
        {
            string strToReturn = url;
            if (url.IndexOf("~/") == 0)
            {
                string path = "http://" + HttpContext.Current.Request.Url.Authority + HttpRuntime.AppDomainAppVirtualPath + "/";

                strToReturn = url.Replace("~/", path);
            }

            return strToReturn;
        }

        /// <summary>
        /// Function to get cssClass
        /// </summary>
        /// <param name="cssClassName">Name of css class</param>
        /// <param name="style">style</param>
        /// <returns>css class</returns>
        private string GetCssClass(string cssClassName, TableItemStyle style)
        {
            if (style.IsEmpty) return string.Empty;



            StringBuilder strToReturn = new StringBuilder();
            strToReturn.Append("." + cssClassName);
            strToReturn.Append("{");

            CssStyleCollection styleCollection = style.GetStyleAttributes(this);
            if (styleCollection.Count <= 0) return string.Empty;

            strToReturn.Append(styleCollection.Value);

            // HORIZONTAL ALIGN
            if (style.HorizontalAlign != HorizontalAlign.NotSet)
            {
                strToReturn.Append("text-align:" + style.HorizontalAlign.ToString() + ";");
            }

            //foreach (string key in styleCollection.Keys)
            //{
            //    string value = styleCollection[key];
            //    strToReturn.Append(key + ":" + value + ";");
            //}

            strToReturn.Append("}");

            return strToReturn.ToString();


        }

        /// <summary>
        /// Function to get HTML color code
        /// </summary>
        /// <param name="color">color</param>
        /// <returns>Html color code</returns>
        private string GetHtmlColor(Color color)
        {
            return ColorTranslator.ToHtml(color);
        }

        #endregion
    }
}
