using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace ImageGallery
{

   // [TypeConverter(typeof(ExpandableObjectConverter))]
    public class Image
    {

        #region DataMembers

        string _name = string.Empty;
        string _imageUrl = string.Empty;
        string _title = string.Empty;
        string _description = string.Empty;
        string _value = string.Empty;
        string _containerID = string.Empty;
        string _navigationBarItemID = string.Empty; 

        bool _selected = false;
        int _index = -1;

        #endregion

        #region Public Propeties
        
        /// <summary>
        /// Get or Set the Name of the Image.
        /// </summary>
        [NotifyParentProperty(true)] 
        [Description("Name of the Image without path.")]
        public string Name
        {
            get { return (_name != null) ? _name : string.Empty; }
            set { _name = value; }
        }

        /// <summary>
        /// Get or Set the Url of the Image.
        /// </summary>
        [NotifyParentProperty(true)] 
        [Description("Url of the image.")]
        [Editor(typeof(System.Web.UI.Design.ImageUrlEditor),typeof(System.Drawing.Design.UITypeEditor))]  
        public string ImageUrl
        {
            get { return (_imageUrl != null) ? _imageUrl : string.Empty; }
            set { _imageUrl = value; }
        }

        /// <summary>
        /// Get or Set the Title of the image.
        /// </summary>
        [NotifyParentProperty(true)] 
        [Description("Title of the image.")]
        public string Title
        {
            get { return (_title != null) ? _title : string.Empty; }
            set { _title = value; }
        }

        /// <summary>
        /// Get or Set the Description of the Image.
        /// </summary>
        [NotifyParentProperty(true)]
        [Description("Description of the image.")]
        public string Description
        {
            get { return (_description != null) ? _description : string.Empty; }
            set { _description = value; }
        }

        /// <summary>
        /// Get or Set the key value the image.
        /// </summary>
        [NotifyParentProperty(true)] 
        [Description("value of image")]
        public string Value
        {
            get { return (_value != null) ? _value : string.Empty; }
            set { _value = value; }
        }

        /// <summary>
        /// Get or Set the is image selected one.
        /// </summary>
        [NotifyParentProperty(true)] 
        [Description("return true of it is current selected image.")]
        public bool Selected
        {
            get { return _selected; }
            set { _selected = value; }
        }

        /// <summary>
        /// Gets the index of image.
        /// </summary>
        [DesignerSerializationVisibility( DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Description("Index of selected Image.")]
        public int Index
        {
            get { return _index; }
            internal set { _index = value; }
        }

        #endregion

        #region Internal Properties


        /// <summary>
        /// Gets the index of image.
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Description("Image Container ID.")]
        internal string ContainerID
        {
            get { return _containerID; }
            set { _containerID = value; }
        }

        /// <summary>
        /// Gets the index of image.
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Description("Navigation Bar Item ID")]
        internal string NavigationBarItemID
        {
            get { return _navigationBarItemID; }
            set { _navigationBarItemID = value; }
        }

        #endregion
    }
}
